/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016-2020 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.office.document;

import java.util.zip.ZipException;

import org.apache.commons.lang3.StringUtils;

public class ZipExceptionHandler {
	private static final String ZIP_PREFIX_INVALID_ENTRY_CRC = "invalid entry CRC";

	private ZipExceptionHandler() {
		// nothing to do
	}

	/**
	 * Interpret a ZipException instance and decide if the exception is
	 * fatal for further processing or not.
	 *
	 * @param e an arbitrary ZipException
	 * @return TRUE if we should not further process the Zip stream and FALSE
	 * if we should ignore the exception.
	 */
	public static boolean isFatalZipError(ZipException e) {
		return isCRCError(e) ? false : true;
	}

	/**
	 * Determines if the ZipException describes a CRC error.
	 *
	 * @param e an arbitrary ZipException
	 * @return TRUE if the ZipException instance describes a CRC error,
	 * otherwise FALSE
	 */
	public static boolean isCRCError(ZipException e) {
		final String msg = e.getMessage();
		if (StringUtils.isNotEmpty(msg) && msg.startsWith(ZIP_PREFIX_INVALID_ENTRY_CRC, 0)) {
			return true;
		}

		return false;
	}
}
