/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016-2020 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.office.filter.core.spreadsheet;

import java.util.List;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import com.openexchange.office.filter.api.OCKey;

public final class SmlUtils {

    public static String getAddressWithoutBraces(String in) {
        if(in==null||in.length()<2) {
            return in;
        }
        if(in.charAt(0)=='['&&in.charAt(in.length()-1)==']') {
            return in.substring(1, in.length()-1);
        }
        return in;
    }

    public static class CellRefRange implements Cloneable {

        final CellRef start;
        final CellRef end;

        public CellRefRange() {
        	start = new CellRef(0, 0);
        	end = new CellRef(0, 0);
        }
        public CellRefRange(int c1, int r1, int c2, int r2) {
        	start = new CellRef(c1, r1);
        	end = new CellRef(c2, r2);
        }
        public CellRefRange(CellRef start, CellRef end) {
            this.start = start;
            this.end = end;
        }
        public CellRef getStart() {
            return start;
        }
        public CellRef getEnd() {
            return end;
        }
        public int getColumns() {
            return (end.getColumn() - start.getColumn()) + 1;
        }
        public int getRows() {
            return (end.getRow() - start.getRow()) + 1;
        }
        public boolean equals(CellRefRange cellRefRange) {
            return start.equals(cellRefRange.getStart())&&end.equals(cellRefRange.getEnd());
        }
        public boolean isInside(int column, int row) {
            return column>=Math.min(start.getColumn(), end.getColumn())&&column<=Math.max(start.getColumn(), end.getColumn())
                && row>=Math.min(start.getRow(), end.getRow())&&row<=Math.max(start.getRow(), end.getRow());
        }
        public boolean intersects(CellRefRange cellRefRange) {

            final int left  = Math.max( start.getColumn(), cellRefRange.getStart().getColumn() );
            final int right = Math.min( end.getColumn(), cellRefRange.getEnd().getColumn() );
            final int top   = Math.max( start.getRow(), cellRefRange.getStart().getRow() );
            final int bottom= Math.min( end.getRow(), cellRefRange.getEnd().getRow() );
            return (right<left||bottom<top) ? false : true;
        }
        public boolean contains(CellRefRange cellRefRange) {
        	return isInside(cellRefRange.getStart().getColumn(), cellRefRange.getStart().getRow())&&
        			isInside(cellRefRange.getEnd().getColumn(), cellRefRange.getEnd().getRow());
        }
        public final JSONObject getJSONObject() throws JSONException {
            final JSONObject jsonObject = new JSONObject();
            jsonObject.put(OCKey.START.value(), start.getJSONArray());
            if (!start.equals(end)) { jsonObject.put(OCKey.END.value(), end.getJSONArray()); }
            return jsonObject;
        }
        public static CellRefRange insertRowRange(CellRefRange cellRefRange, int startRow, int count, boolean append) {
            final CellRef start = cellRefRange.getStart();
            final CellRef end = cellRefRange.getEnd();
            int r1 = start.getRow();
            int r2 = end.getRow();
            if(startRow<=start.getRow()) {
                r1+=count;
                r2+=count;
            }
            else if(startRow<=(append?end.getRow()+1:end.getRow())) {
                r2+=count;
            }
            if(r2>1048575) {
                r2=1048575;
                if(r1>1048575) {
                    return null;
                }
            }
            return new CellRefRange(new CellRef(start.getColumn(), r1), new CellRef(end.getColumn(), r2));
        }
        public static CellRefRange deleteRowRange(CellRefRange cellRef, int startRow, int count) {
        	int endRow = (startRow + count)-1;
        	int row1 = cellRef.getStart().getRow();
            int row2 = cellRef.getEnd().getRow();
            if(startRow<=row2) {
                if(startRow<=row1&&endRow>=row2) {
                    return null;                                // merged cell is completely removed
                }
                else if(endRow<row1) {
                    row1 -= count;                              // merged cell is completely moved up by count
                    row2 -= count;
                }
                else if(startRow>row1&&endRow<=row2) {
                    row2 -= count;                              // merged cell is shortened by delete count
                }
                else if(startRow>row1) {
                    row2 = startRow-1;                          // new end is to be set
                }
                else {
                    row1 = startRow;                            // new start is to be set
                    row2 -= count;
                }
            }
            return new CellRefRange(new CellRef(cellRef.getStart().getColumn(), row1), new CellRef(cellRef.getEnd().getColumn(), row2));
        }
        public static CellRefRange insertColumnRange(CellRefRange cellRefRange, int startColumn, int count, boolean append) {
            final CellRef start = cellRefRange.getStart();
            final CellRef end = cellRefRange.getEnd();
            int c1 = start.getColumn();
            int c2 = end.getColumn();
            if(startColumn<=start.getColumn()) {
                c1+=count;
                c2+=count;
            }
            else if(startColumn<=(append?end.getColumn()+1:end.getColumn())) {
                c2+=count;
            }
            if(c2>16383) {
                c2=16383;
                if(c1>16383) {
                    return null;
                }
            }
            return new CellRefRange(new CellRef(c1, start.getRow()), new CellRef(c2, end.getRow()));
        }
        public static CellRefRange deleteColumnRange(CellRefRange cellRef, int startColumn, int count) {
        	int endColumn = (startColumn+count)-1;
            int column1 = cellRef.getStart().getColumn();
            int column2 = cellRef.getEnd().getColumn();
            if(startColumn<=column2) {
                if(startColumn<=column1&&endColumn>=column2) {
                    return null;                                // merged cell is completely removed
                }
                else if(endColumn<column1) {
                    column1 -= count;                           // merged cell is completely moved up by count
                    column2 -= count;
                }
                else if(startColumn>column1&&endColumn<=column2) {
                    column2 -= count;                           // merged cell is shortened by delete count
                }
                else if(startColumn>column1) {
                    column2 = startColumn-1;                    // new end is to be set
                }
                else {
                    column1 = startColumn;                      // new start is to be set
                    column2 -= count;
                }
            }
            return new CellRefRange(new CellRef(column1, cellRef.getStart().getRow()), new CellRef(column2, cellRef.getEnd().getRow()));
        }

        @Override
        public CellRefRange clone() {
            return new CellRefRange(start.clone(), end.clone());
        }

        @Override
        public String toString() {
            return "CellRefRange [start=" + start + ", end=" + end + "]";
        }

        // if ignoreFullRange is not null the range is not changed if the range length is as long as ignoreFullRange
        public static void insertRows(List<? extends ICellRefRangeAccess> list, int start, int count, boolean append, Integer ignoreFullRange) {
            for(int i=0;i<list.size();i++) {
                final ICellRefRangeAccess refRangeAccess = list.get(i);
                final CellRefRange cellRefRange = refRangeAccess.getCellRefRange(true);
                if(ignoreFullRange==null||cellRefRange.getRows()!=ignoreFullRange) {
                    final SmlUtils.CellRefRange newMergeCell = SmlUtils.CellRefRange.insertRowRange(cellRefRange, start, count, append);
                    if(newMergeCell==null) {
                        list.remove(i--);
                    }
                    else {
                        refRangeAccess.setCellRefRange(newMergeCell);
                    }
                }
            }
        }

        // if ignoreFullRange is not null the range is not changed if the range length is as long as ignoreFullRange
        public static void deleteRows(List<? extends ICellRefRangeAccess> list, int start, int count, boolean removeSingleRange, Integer ignoreFullRange) {
            for(int i=0;i<list.size();i++) {
                final ICellRefRangeAccess refRangeAccess = list.get(i);
                final CellRefRange cellRefRange = refRangeAccess.getCellRefRange(true);
                if(ignoreFullRange==null||cellRefRange.getRows()!=ignoreFullRange) {
                    final SmlUtils.CellRefRange newMergeCell = SmlUtils.CellRefRange.deleteRowRange(cellRefRange, start, count);
                    if(newMergeCell==null) {
                        list.remove(i--);
                    }
                    else if(removeSingleRange&&newMergeCell.getStart().equals(newMergeCell.getEnd())) {
                        list.remove(i--);
                    }
                    else {
                        refRangeAccess.setCellRefRange(newMergeCell);
                    }
                }
            }
        }

        // if ignoreFullRange is not null the range is not changed if the range length is as long as ignoreFullRange
        public static void insertColumns(List<? extends ICellRefRangeAccess> list, int start, int count, boolean append, Integer ignoreFullRange) {
            for(int i=0;i<list.size();i++) {
                final ICellRefRangeAccess refRangeAccess = list.get(i);
                final CellRefRange cellRefRange = refRangeAccess.getCellRefRange(true);
                if(ignoreFullRange==null||cellRefRange.getColumns()!=ignoreFullRange) {
                    final SmlUtils.CellRefRange newMergeCell = SmlUtils.CellRefRange.insertColumnRange(cellRefRange, start, count, append);
                    if(newMergeCell==null) {
                        list.remove(i--);
                    }
                    else {
                        refRangeAccess.setCellRefRange(newMergeCell);
                    }
                }
            }
        }

        // if ignoreFullRange is not null the range is not changed if the range length is as long as ignoreFullRange
        public static void deleteColumns(List<? extends ICellRefRangeAccess> list, int start, int count, boolean removeSingleRange, Integer ignoreFullRange) {
            for(int i=0;i<list.size();i++) {
                final ICellRefRangeAccess refRangeAccess = list.get(i);
                final CellRefRange cellRefRange = refRangeAccess.getCellRefRange(true);
                if(ignoreFullRange==null||cellRefRange.getColumns()!=ignoreFullRange) {
                    final SmlUtils.CellRefRange newMergeCell = SmlUtils.CellRefRange.deleteColumnRange(cellRefRange, start, count);
                    if(newMergeCell==null) {
                        list.remove(i--);
                    }
                    else if(removeSingleRange&&newMergeCell.getStart().equals(newMergeCell.getEnd())) {
                        list.remove(i--);
                    }
                    else {
                        refRangeAccess.setCellRefRange(newMergeCell);
                    }
                }
            }
        }
    }

    public static class CellRef implements Cloneable {

        private int column;
        private int row;

        public CellRef(int column, int row) {
            this.column = column;
            this.row = row;
        }
        public CellRef(JSONArray ref) {
            try {
                this.column = ref.getInt(0);
                this.row = ref.getInt(1);
            }
            catch (JSONException e) {
                this.column = 0;
                this.row = 0;
            }
        }
        public int getColumn() {
            return column;
        }
        public int getRow() {
            return row;
        }
        public void setColumn(int column) {
            this.column = column;
        }
        public void setRow(int row) {
            this.row = row;
        }

        @Override
        public boolean equals(Object o) {
            if (o instanceof CellRef) {
                CellRef cellRef = (CellRef) o;
                return this.column == cellRef.column && this.row == cellRef.row;

            }
            return false;
        }

        public final JSONArray getJSONArray() {
            final JSONArray jsonArray = new JSONArray(2);
            jsonArray.put(column);
            jsonArray.put(row);
            return jsonArray;
        }
        @Override
        public CellRef clone() {
            try {
                return (CellRef)super.clone();
            } catch (CloneNotSupportedException e) {
                return null;
            }
        }

        @Override
        public String toString() {
            return "CellRef [column=" + column + ", row=" + row + "]";
        }

        public static boolean insertRows(CellRef cellRef, int start, int count) {
            if(cellRef!=null) {
                final int r = cellRef.getRow();
                final int to = r + count;
                if (r>=1048576) {
                    return true;
                } else if (r>=start) {
                    cellRef.setRow(to);
                }
            }
            return false;
        }
        public static void insertRows(List<? extends ICellRefAccess> list, int start, int count) {
            for(int i=0;i<list.size();i++) {
                if(insertRows(list.get(i).getCellRef(true), start, count)) {
                    list.remove(i--);
                }
            }
        }

        public static boolean deleteRows(CellRef cellRef, int start, int count) {
            if(cellRef!=null) {
                final int end = start+count-1;
                final int r = cellRef.getRow();
                if (r>end) {
                    cellRef.setRow(r - count);
                } else if (r>=start) {
                    return true;
                }
            }
            return false;
        }
        public static void deleteRows(List<? extends ICellRefAccess> list, int start, int count) {
            for(int i=0;i<list.size();i++) {
                if(deleteRows(list.get(i).getCellRef(true), start, count)) {
                    list.remove(i--);
                }
            }
        }

        public static boolean insertColumns(CellRef cellRef, int start, int count) {
            if(cellRef!=null) {
                final int c = cellRef.getColumn();
                final int to = c + count;
                if (c>=16383) {
                    return true;
                } else if (c>=start) {
                    cellRef.setColumn(to);
                }
            }
            return false;
        }
        public static void insertColumns(List<? extends ICellRefAccess> list, int start, int count) {
            for(int i=0;i<list.size();i++) {
                if(insertColumns(list.get(i).getCellRef(true), start, count)) {
                    list.remove(i--);
                }
            }
        }

        public static boolean deleteColumns(CellRef cellRef, int start, int count) {
            if(cellRef!=null) {
                final int end = start+count-1;
                final int c = cellRef.getColumn();
                if (c>end) {
                    cellRef.setColumn(c - count);
                } else if (c>=start) {
                    return true;
                }
            }
            return false;
        }
        public static void deleteColumns(List<? extends ICellRefAccess> list, int start, int count) {
            for(int i=0;i<list.size();i++) {
                if(deleteColumns(list.get(i).getCellRef(true), start, count)) {
                    list.remove(i--);
                }
            }
        }
    }

    public static CellRefRange createCellRefRange(String range) {
        if(range==null)
            return null;
        final CellRef start = createCellRef(range, 0, 0);
        if(start==null)
            return null;
        int indexOfSeparator = range.indexOf(':');
        if(indexOfSeparator==-1) {
            return new CellRefRange(start, new CellRef(start.getColumn(), start.getRow()));
        }
        if(indexOfSeparator<2||(indexOfSeparator+1)>=range.length())
            return null;
        final CellRef end = createCellRef(range.substring(indexOfSeparator+1), 16383, 1048575);
        if(end==null)
            return null;
        return new CellRefRange(start, end);
    }

    public static CellRefRange[] createCellRefRanges(String ranges) {
        if(ranges==null) {
            return new CellRefRange[0];
        }
        final String[] tokenArray = ranges.split(" ", -1);
        final CellRefRange[] cellRefRangeArray = new CellRefRange[tokenArray.length];
        for(int i = 0; i < tokenArray.length; i++) {
            cellRefRangeArray[i] = createCellRefRange(tokenArray[i]);
        }
        return cellRefRangeArray;
    }

    /*
     * odf uses "." whereas xlsx is using "!" as separator
     */
    public static CellRef createCellRefWithSheet(String cellRef, String sheetSeparator) {
        final int indexOf = cellRef.lastIndexOf(sheetSeparator);
        return createCellRef(indexOf!=-1 ? cellRef.substring(indexOf + 1) : cellRef);
    }

    public static CellRefRange createCellRefRangeWithSheet(String cellRefRange, String sheetSeparator) {
        final int indexOf = cellRefRange.lastIndexOf(sheetSeparator);
        final String range = cellRefRange.substring(indexOf + 1);
        return createCellRefRange(range);
    }

    /**
     * translates string CellRef like "B23" into a computable form : 1, 22
     *
     * returns null on error or missing cellRef
     *
     */
    public static CellRef createCellRef(String cellRef) {
    	return createCellRef(cellRef, null, null);
    }

    /**
     * translates string CellRef like "B23" into a computable form : 1, 22
     *
     * returns null on error or missing cellRef
     *
     */
    public static CellRef createCellRef(String cellRef, Integer defaultX, Integer defaultY) {

        if(cellRef==null||cellRef.isEmpty())
            return null;

        CellRef cellPosition = null;
        Integer x = null;
        Integer y = null;

        int characters = 0;
        int characterStartIndex = cellRef.charAt(0) == '$' ? 1 : 0;

        while((characters+characterStartIndex)<cellRef.length()) {
            char character = cellRef.charAt(characterStartIndex+characters);
            if((character<'A')||(character>'Z')) {
                break;
            }
            characters++;
        }
        if(characters==0&&defaultX!=null) {
        	x = defaultX;
        }
        else if(characters>0&&characters<6) {

            final int[] exMap = { 1, 26, 676, 17576, 456976 };

            int ex = characters - 1;
            x = -1;

            for(int i = 0; i < characters; i++) {
                x +=((cellRef.charAt(i+characterStartIndex)-'A')+1)*exMap[ex-i];
            }
        }

        if(x==null) {
        	return null;
        }

        int numbers = 0;
        int numberStartIndex = characterStartIndex += characters;

        if(numberStartIndex<cellRef.length()&&cellRef.charAt(numberStartIndex) == '$') {
            numberStartIndex++;
        }
        while(numbers+numberStartIndex<cellRef.length()) {
            char character = cellRef.charAt(numberStartIndex+numbers);
            if((character<'0')||(character>'9')) {
                break;
            }
            numbers++;
        }
        if(numbers==0&&defaultY!=null) {
        	y = defaultY;
        }
        else if(numbers>0&&numbers<=7) {
        	y = 0;
        	for(int i = 0; i < numbers; i++) {
                y *= 10;
                y += cellRef.charAt(i+numberStartIndex)-'0';
            }
        	y--;
        }
        if(y!=null) {
            cellPosition = new CellRef(x, y);
        }
        return cellPosition;
    }

    private static void addCellRef(StringBuilder buffer, int column, long row) {
        addCellRefColumn(buffer, column);
        addCellRefRow(buffer, row);
    }

    private static void addCellRefColumn(StringBuilder buffer, int column) {

        int pos = buffer.length();
        while(column>=0) {
            buffer.insert(pos, (char)('A' + (column % 26)));
            column /= 26;
            column--;
        }
    }

    private static void addCellRefRow(StringBuilder buffer, long row) {
        buffer.append(Long.toString(row+1));
    }

    public static String getCellRef(CellRef cellRef) {
        return getCellRef(cellRef.getColumn(), cellRef.getRow());
    }
    public static String getCellRef(int column, long row) {
        StringBuilder buffer = new StringBuilder(8);
        addCellRef(buffer, column, row);
        return buffer.toString();
    }

    public static String getCellRefRange(CellRefRange cellRefRange) {
        return getCellRefRange(cellRefRange.getStart().getColumn(), cellRefRange.getStart().getRow(), cellRefRange.getEnd().getColumn(), cellRefRange.getEnd().getRow());
    }

    public static String getCellRefRange(JSONObject range) throws JSONException {
        final JSONArray start = range.getJSONArray(OCKey.START.value());
        final JSONArray end = range.optJSONArray(OCKey.END.value());
        final SmlUtils.CellRef startRef = new SmlUtils.CellRef(start.getInt(0), start.getInt(1));
        final SmlUtils.CellRef endRef = (end==null) ? startRef : new SmlUtils.CellRef(end.getInt(0), end.getInt(1));
        return getCellRefRange(new SmlUtils.CellRefRange(startRef, endRef));
    }

    public static String getAbsoluteCellRefRange(CellRefRange cellRefRange) {
        return getAbsoluteCellRefRange(cellRefRange.getStart().getColumn(), cellRefRange.getStart().getRow(), cellRefRange.getEnd().getColumn(), cellRefRange.getEnd().getRow());
    }

    public static String getCellRefRange(int startColumn, long startRow, int endColumn, int endRow) {
        StringBuilder buffer = new StringBuilder(17);
        addCellRef(buffer, startColumn, startRow);
        buffer.append(':');
        addCellRef(buffer, endColumn, endRow);
        return buffer.toString();
    }
    public static String getAbsoluteCellRefRange(int startColumn, long startRow, int endColumn, int endRow) {
        StringBuilder buffer = new StringBuilder(21);
        buffer.append('$');
        addCellRefColumn(buffer, startColumn);
        buffer.append('$');
        addCellRefRow(buffer, startRow);
        buffer.append(':');
        buffer.append('$');
        addCellRefColumn(buffer, endColumn);
        buffer.append('$');
        addCellRefRow(buffer, endRow);
        return buffer.toString();
    }
}
