/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016-2020 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

/**
 * @author sven.jacobi@open-xchange.com
 */

package com.openexchange.office.filter.odf.draw;

import java.util.Map;
import org.json.JSONException;
import org.json.JSONObject;
import com.openexchange.office.filter.api.OCKey;
import com.openexchange.office.filter.odf.AttributesImpl;
import com.openexchange.office.filter.odf.OdfOperationDoc;
import com.openexchange.office.filter.odf.OpAttrs;

/*
 * in contrast to normal shapes the line is using "svg:x1, svg:y1, svg:y2, svg:x2" instead of "svg:x, svg:y, svg:width and svg:height" :(
 */

public class LineShape extends Shape {

    public LineShape(AttributesImpl attributes, String uri, String localName, String qName, GroupShape parentGroup, boolean rootShape, boolean contentStyle) {
	    super(attributes, uri, localName, qName, parentGroup, rootShape, contentStyle);
	}

	public LineShape(OdfOperationDoc operationDoc, String uri, String localName, String qName, GroupShape parentGroup, boolean rootShape, boolean contentStyle) {
	    super(operationDoc, uri, localName, qName, parentGroup, rootShape, contentStyle);
	}

    @Override
    public DrawingType getType() {
        return DrawingType.SHAPE;
    }

	@Override
	public boolean isLineMode() {
	    return true;
	}

    @Override
    public void applyAttrsFromJSON(OdfOperationDoc operationDocument, JSONObject attrs, boolean contentAutoStyle)
        throws JSONException {

        super.applyAttrsFromJSON(operationDocument, attrs, contentAutoStyle);

        final JSONObject drawingAttrs = attrs.optJSONObject(OCKey.DRAWING.value());
        if(drawingAttrs!=null) {
            final Object flipH = drawingAttrs.opt(OCKey.FLIP_H.value());
            if(flipH!=null) {
                double width = transformer.getWidth();
                double x = transformer.getX();
                final boolean flip;
                if(flipH==JSONObject.NULL) {
                    flip = width < 0 ;
                }
                else {
                    flip = ((Boolean)flipH).booleanValue();
                }
                if(flip) {
                    transformer.setWidth(-width);
                    transformer.setX(x + width);
                }
            }
            final Object flipV = drawingAttrs.opt(OCKey.FLIP_V.value());
            if(flipV!=null) {
                double height = transformer.getHeight();
                double y = transformer.getY();
                final boolean flip;
                if(flipV==JSONObject.NULL) {
                    flip = height < 0;
                }
                else {
                    flip = ((Boolean)flipV).booleanValue();
                }
                if(flip) {
                    transformer.setHeight(-height);
                    transformer.setY(y + height);
                }
            }
        }
    }

    @Override
    public void createAttrs(OdfOperationDoc operationDocument, OpAttrs attrs, boolean contentAutoStyle) {
        super.createAttrs(operationDocument, attrs, contentAutoStyle);

        final Map<String, Object> geometryAttrs = attrs.getMap(OCKey.GEOMETRY.value(), true);
        geometryAttrs.put(OCKey.PRESET_SHAPE.value(), "line");
    }
}
