/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016-2020 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

/**
 * @author sven.jacobi@open-xchange.com
 */

package com.openexchange.office.filter.odf.listlevel;

import org.apache.xml.serializer.SerializationHandler;
import org.json.JSONObject;
import org.xml.sax.SAXException;
import com.openexchange.office.filter.odf.AttributesImpl;
import com.openexchange.office.filter.odf.Namespaces;
import com.openexchange.office.filter.odf.OpAttrs;
import com.openexchange.office.filter.odf.SaxContextHandler;
import com.openexchange.office.filter.odf.properties.StylePropertiesBase;
import com.openexchange.office.filter.odf.styles.StyleManager;

public class ListLevelProperties extends StylePropertiesBase {

	private ListLevelLabelAlignment listLevelLabelAlignment = null;

	public ListLevelProperties(AttributesImpl attributesImpl) {
		super(attributesImpl);
	}

    public ListLevelLabelAlignment getListLevelLabelAlignment(boolean forceCreate) {
        if(listLevelLabelAlignment==null&&forceCreate) {
            listLevelLabelAlignment = new ListLevelLabelAlignment(new AttributesImpl());
        }
        return listLevelLabelAlignment;
    }

    public void setListLevelLabelAlignment(ListLevelLabelAlignment listLevelLabelAlignment) {
        this.listLevelLabelAlignment = listLevelLabelAlignment;
    }

    static public boolean hasData(ListLevelProperties o) {
        if(o==null) {
            return false;
        }
        if(!o.getAttributes().isEmpty()) {
            return true;
        }
        return ListLevelLabelAlignment.hasData(o.listLevelLabelAlignment);
    }

    @Override
    public void writeObject(SerializationHandler output)
        throws SAXException {

        if(!attributes.isEmpty()||(getContent()!=null&&!getContent().isEmpty())||!getTextContent().isEmpty()) {
            SaxContextHandler.startElement(output, getNamespace(), getLocalName(), getQName());
            attributes.write(output);
            if(!text.isEmpty()) {
                output.characters(text);
            }
            if(ListLevelLabelAlignment.hasData(listLevelLabelAlignment)) {
                listLevelLabelAlignment.writeObject(output);
            }
            SaxContextHandler.endElement(output, getNamespace(), getLocalName(), getQName());
        }
    }

    @Override
    public int hashCode() {
        int hash = super.hashCode();
        if(ListLevelLabelAlignment.hasData(listLevelLabelAlignment)) {
            hash = hash * 31 + listLevelLabelAlignment.hashCode();
        }
        return hash;
    }

    @Override
    public boolean equals(Object obj) {
        if(!super.equals(obj)) {
            return false;
        }
        final ListLevelProperties other = (ListLevelProperties)obj;
        if(ListLevelLabelAlignment.hasData(listLevelLabelAlignment)) {
            if(!ListLevelLabelAlignment.hasData(other.listLevelLabelAlignment)) {
                return false;
            }
            if(!listLevelLabelAlignment.equals(other.listLevelLabelAlignment)) {
                return false;
            }
        }
        else {
            if(ListLevelLabelAlignment.hasData(other.listLevelLabelAlignment)) {
                return false;
            }
        }
        return true;
    }

	@Override
	public ListLevelProperties clone() {
		final ListLevelProperties clone = (ListLevelProperties)super._clone();
		if(listLevelLabelAlignment!=null) {
		    clone.setListLevelLabelAlignment(listLevelLabelAlignment.clone());
		}
		return clone;
	}

	@Override
	public void applyAttrs(StyleManager styleManager, JSONObject attrs) {
		//
	}

	@Override
	public void createAttrs(StyleManager styleManager, boolean contentAutoStyle, OpAttrs attrs) {
		// TODO Auto-generated method stub
	}

	@Override
	public String getQName() {
		return "style:list-level-properties";
	}

	@Override
	public String getLocalName() {
		return "list-level-properties";
	}

	@Override
	public String getNamespace() {
		return Namespaces.STYLE;
	}
}
