/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016-2020 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

/**
 * @author sven.jacobi@open-xchange.com
 */

package com.openexchange.office.filter.odf.styles;

import java.util.Currency;
import java.util.Iterator;
import java.util.Locale;
import java.util.Map;
import org.json.JSONObject;
import org.odftoolkit.odfdom.IElementWriter;
import com.openexchange.office.filter.odf.AttributesImpl;
import com.openexchange.office.filter.odf.Namespaces;
import com.openexchange.office.filter.odf.OpAttrs;
import com.openexchange.office.filter.odf.properties.NumberCurrencySymbol;
import com.openexchange.office.filter.odf.properties.NumberNumber;
import com.openexchange.office.filter.odf.properties.NumberText;
import com.openexchange.office.filter.odf.properties.PropertyHelper;
import com.openexchange.office.filter.odf.properties.TextProperties;

public class NumberCurrencyStyle extends NumberStyleBase {

	public NumberCurrencyStyle(String name, boolean automaticStyle, boolean contentStyle) {
		super(null, name, automaticStyle, contentStyle);
	}

	public NumberCurrencyStyle(String name, AttributesImpl attributesImpl, boolean automaticStyle, boolean contentStyle) {
		super(name, attributesImpl, false, automaticStyle, contentStyle);
	}

	@Override
	public String getQName() {
		return "number:currency-style";
	}

	@Override
	public String getLocalName() {
		return "currency-style";
	}

	@Override
	public String getNamespace() {
		return Namespaces.NUMBER;
	}

	@Override
	public String getFormat(StyleManager styleManager, Map<String, String> additionalStyleProperties, boolean contentAutoStyle) {
		String result = "";
		final Iterator<IElementWriter> propertiesIter = getContent().iterator();
		while(propertiesIter.hasNext()) {
			final IElementWriter properties = propertiesIter.next();
			if (properties instanceof NumberCurrencySymbol) {
				final NumberCurrencySymbol numberCurrencySymbol = (NumberCurrencySymbol)properties;
			    result += "[$";
				result += numberCurrencySymbol.getTextContent();
				final String language = numberCurrencySymbol.getAttribute("number:language");
				final String country = numberCurrencySymbol.getAttribute("number:country");
                if(language!=null&&!language.isEmpty() ) {
                    result += "-";
                    //convert local to ms language id
                    result += PropertyHelper.getMSLangCode(language, country);
                }
                result += "]";
			} else if (properties instanceof NumberNumber) {
				result += ((NumberNumber)properties).getNumberFormat();
			} else if (properties instanceof NumberText) {
				String textcontent = ((NumberText)properties).getTextContent();
				if (textcontent == null || textcontent.length() == 0) {
					textcontent = " ";
				}
				result += textcontent;
            } else if (properties instanceof TextProperties) {
                result += ((TextProperties)properties).getColorFromElement();
            }
		}
		if(!getMapStyleList().isEmpty()) {
            result = getMapStyleList().get(0).getMapping(styleManager, contentAutoStyle, getFamily()) + ";" + result;
		}
		return result;
	}

	@Override
	public void setFormat(String formatCode, Map<String, String> additionalStyleProperties) {

        String currentTextToken = "";
        for(int pos = 0; pos < formatCode.length(); ++pos){
            char c = formatCode.charAt(pos);
            if(c == '"'){
                //add all characters until the next quotation to the current text token
                currentTextToken += c;
                while(pos < formatCode.length() - 1  ){
                    c = formatCode.charAt(pos + 1);
                    if(c == '\\'){
                        currentTextToken += c;
                        if((pos > formatCode.length() - 2))
                            break; //invalid!
                        currentTextToken += c;
                        currentTextToken += formatCode.charAt(pos + 2);
                        ++pos;
                    } else if(c == '"' ){
                        currentTextToken += c;
                        ++pos;
                        break;
                    } else {
                        currentTextToken += c;
                    }
                    ++pos;
                }
            } else if(c == '['){
                if(!currentTextToken.isEmpty()){
                	getContent().add(new NumberText(currentTextToken));
                    currentTextToken = "";
                }
                int closePos = formatCode.indexOf(']', pos);
                if(closePos < 0 || closePos < pos + 2){ // minimum two characters inside, closing bracket not quoted
                    break; // invalid
                }
                String bracketToken = formatCode.substring(pos, closePos + 1);
                if(getContent().isEmpty() && !getColorElement(bracketToken.substring(1, bracketToken.length() - 1)).isEmpty()){
                    String color = getColorElement(bracketToken.substring(1, bracketToken.length() - 1));
                    if(!color.isEmpty()) {
                    	final TextProperties textProperties = new TextProperties(new AttributesImpl());
                    	textProperties.getAttributes().setValue(Namespaces.FO, "color", "fo:color", color);
                    	getContent().add(textProperties);
                    }
                } else if(bracketToken.charAt(1) == '$'){
                	emitCurrency(bracketToken);
                }
                pos = closePos;

            } else if (c == '#' || c == '0' || c == '.') {
                if(!currentTextToken.isEmpty()){
                	getContent().add(new NumberText(currentTextToken));
                    currentTextToken = "";
                }
                final int numPos = pos;
                while(++pos < formatCode.length()){
                    // TODO: hashes can only be at the beginning interrupted by comma
                    c = formatCode.charAt(pos);
                    if((c != '#' && c != '.' && c != ',' && c != '0') || (pos == (formatCode.length() - 1))) {
                        String numberToken = formatCode.substring(numPos, pos + 1);
                        if(numberToken.charAt(numberToken.length() - 1) == '.'){
                            //add replacement chars
                            int spacePos = formatCode.indexOf(' ', pos-1);
                            int bracketPos = formatCode.indexOf('[', pos-1);
                            if(spacePos < 0) {
                                spacePos = bracketPos;
                            } else if(bracketPos < 0){
                                bracketPos = spacePos;
                            }
                            spacePos = Math.min(spacePos, bracketPos);
                            if(spacePos < 0){ // if not found add the rest of the format TODO: Are there other delimiters than space and bracket? ?
                                spacePos = formatCode.length();
                            }
                            if(spacePos > 0){
                                numberToken += formatCode.substring(pos, spacePos );
                                pos = spacePos;
                            }
                        }
                        // workaround for Bug 48866
                        numberToken = numberToken.replaceAll("[\\[\\s]", "");
                        emitNumber(numberToken);
                        break;
                    }
                }
                if (c == '[') {
                    pos--; // workaround for Bug 48866
                }
            } else if( c == '\\'){
                //add this and the next character to the text token
                currentTextToken += c;
                if(pos > formatCode.length() - 2){
                    break;//invalid
                }
                currentTextToken += formatCode.charAt(pos + 1);
                ++pos;
            } else {
                currentTextToken += c;
            }
        }
        if(!currentTextToken.isEmpty()){
        	getContent().add(new NumberText(currentTextToken));
        }
	}

	public String getCurrencyCode() {
		String currencyValue = null;

		final Iterator<IElementWriter> propertiesIter = getContent().iterator();
		while(propertiesIter.hasNext()) {
			final IElementWriter properties = propertiesIter.next();
			if(properties instanceof NumberCurrencySymbol) {
				currencyValue = ((NumberCurrencySymbol)properties).getTextContent();
				if(currencyValue!=null&&currencyValue.equals("DM")) {
					return "DEM";
				}
				final String language =  ((NumberCurrencySymbol)properties).getAttribute("number:language");
				final String country = ((NumberCurrencySymbol)properties).getAttribute("number:country");
				if(language!=null) {
					try {
						final Currency currency = Currency.getInstance(country!=null ? new Locale(language, country) : new Locale(language));
						if(currency!=null) {
							return currency.toString();
						}
					}
					catch(IllegalArgumentException e) {
					    //
					}
				}
				break;
			}
		}
		return currencyValue;
	}

    protected void emitNumber(String numberToken){
        final NumberNumber number = new NumberNumber(new AttributesImpl());
        /* Process part before the decimal point (if any) */
        int nDigits = 0;
        char ch;
        int pos;
        for (pos = 0; pos < numberToken.length()
                && (ch = numberToken.charAt(pos)) != '.'; pos++) {
            if (ch == ',') {
            	number.getAttributes().setBooleanValue(Namespaces.NUMBER, "grouping", "number:grouping", true);
            } else if (ch == '0') {
                nDigits++;
            }
        }
        number.getAttributes().setIntValue(Namespaces.NUMBER, "min-integer-digits", "number:min-integer-digits", nDigits);

        /* Number of decimal places is the length after the decimal */
        if (pos < numberToken.length()) {
        	number.getAttributes().setIntValue(Namespaces.NUMBER, "decimal-places", "number:decimal-places", numberToken.length() - (pos + 1));
            if(pos < numberToken.length() - 1 && numberToken.charAt(pos + 1) != '0'){
            	number.getAttributes().setValue(Namespaces.NUMBER, "decimal-replacement", "number:decimal-replacement", numberToken.substring(pos + 1));
            }
        }
        else {
        	number.getAttributes().setIntValue(Namespaces.NUMBER, "decimal-places", "number:decimal-places", 0);
        }
        getContent().add(number);
    }

    private void emitCurrency(String currencyToken){
        String innerText = currencyToken.substring(1, currencyToken.length() - 1);
        String currencySymbol = "";
        String languageCode = "";
        if(innerText.startsWith("$") && innerText.length() > 1) {
            int dashPos = innerText.indexOf("-");
            currencySymbol = innerText.substring(1, dashPos > 0 ? dashPos : innerText.length());
            if(dashPos > 0){
                languageCode = innerText.substring(dashPos + 1);
            }
        }
        if(!currencySymbol.isEmpty()) {
        	final NumberCurrencySymbol cSymbol = new NumberCurrencySymbol(new AttributesImpl());
            String locale = PropertyHelper.getLocaleFromLangCode(languageCode);
            if(!locale.isEmpty()) {
            	int dashPos = locale.indexOf("-");
                cSymbol.getAttributes().setValue(Namespaces.NUMBER, "language", "number:language", locale.substring(0, dashPos));
                if(dashPos > 0) {
                	cSymbol.getAttributes().setValue(Namespaces.NUMBER, "country", "number:country", locale.substring(dashPos + 1));
                }
            }
            cSymbol.setTextContent(currencySymbol);
            getContent().add(cSymbol);
         }
    }

	@Override
	public void mergeAttrs(StyleBase styleBase) {
	    //
	}

	@Override
	public void applyAttrs(StyleManager styleManager, JSONObject attrs) {
	    //
	}

	@Override
	public void createAttrs(StyleManager styleManager, OpAttrs attrs) {
		// TODO Auto-generated method stub

	}
}
