/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016-2020 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

/**
 * @author sven.jacobi@open-xchange.com
 */

package com.openexchange.office.filter.odf.styles;

import org.apache.xml.serializer.SerializationHandler;
import org.json.JSONObject;
import org.xml.sax.SAXException;
import com.openexchange.office.filter.odf.AttributesImpl;
import com.openexchange.office.filter.odf.OpAttrs;
import com.openexchange.office.filter.odf.SaxContextHandler;
import com.openexchange.office.filter.odf.properties.HeaderFooterProperties;
import com.openexchange.office.filter.odf.properties.PageLayoutProperties;

final public class StylePageLayout extends StyleBase {

	private PageLayoutProperties pageLayoutProperties = new PageLayoutProperties(new AttributesImpl());
	private HeaderFooterProperties headerProperties = new HeaderFooterProperties(new AttributesImpl());
	private HeaderFooterProperties footerProperties = new HeaderFooterProperties(new AttributesImpl());

	public StylePageLayout(String name, boolean automaticStyle, boolean contentStyle) {
		super(null, name, automaticStyle, contentStyle);
	}

	public StylePageLayout(String name, AttributesImpl attributesImpl, boolean defaultStyle, boolean automaticStyle, boolean contentStyle) {
		super(name, attributesImpl, defaultStyle, automaticStyle, contentStyle);
	}

	@Override
    public StyleFamily getFamily() {
		return StyleFamily.PAGE_LAYOUT;
	}

	@Override
    public String getQName() {
		return "style:page-layout";
	}

	@Override
    public String getLocalName() {
		return "page-layout";
	}
	
	public PageLayoutProperties getPageLayoutProperties() {
		return pageLayoutProperties;
	}

	public HeaderFooterProperties getHeaderProperties() {
		return headerProperties;
	}

	public HeaderFooterProperties getFooterProperties() {
		return footerProperties;
	}

	@Override
	public void writeObject(SerializationHandler output)
		throws SAXException {

		SaxContextHandler.startElement(output, getNamespace(), getLocalName(), getQName());
		writeAttributes(output);
		pageLayoutProperties.writeObject(output);
		SaxContextHandler.startElement(output, getNamespace(), "header-style", "style:header-style");
		headerProperties.writeObject(output);
		SaxContextHandler.endElement(output, getNamespace(), "header-style", "style:header-style");
		SaxContextHandler.startElement(output, getNamespace(), "footer-style", "style:footer-style");
		footerProperties.writeObject(output);
		SaxContextHandler.endElement(output, getNamespace(), "footer-style", "style:footer-style");
		SaxContextHandler.endElement(output, getNamespace(), getLocalName(), getQName());
	}

	@Override
	public void mergeAttrs(StyleBase styleBase) {
		if(styleBase instanceof StylePageLayout) {
			getPageLayoutProperties().mergeAttrs(((StylePageLayout)styleBase).getPageLayoutProperties());
			getHeaderProperties().mergeAttrs(((StylePageLayout)styleBase).getHeaderProperties());
			getFooterProperties().mergeAttrs(((StylePageLayout)styleBase).getFooterProperties());
		}
	}

	@Override
	public void applyAttrs(StyleManager styleManager, JSONObject attrs) {
		pageLayoutProperties.applyAttrs(styleManager, attrs);
		headerProperties.applyAttrs(styleManager, attrs);
		footerProperties.applyAttrs(styleManager, attrs);
	}

	@Override
	public void createAttrs(StyleManager styleManager, OpAttrs attrs) {
		pageLayoutProperties.createAttrs(styleManager, isContentStyle(), attrs);
		headerProperties.createAttrs(styleManager, isContentStyle(), attrs);
		footerProperties.createAttrs(styleManager, isContentStyle(), attrs);
	}

	@Override
	protected int _hashCode() {
		int hashCode = pageLayoutProperties.hashCode();
		hashCode = 31 * hashCode + headerProperties.hashCode();
		return 31 * hashCode + footerProperties.hashCode();
	}

	@Override
	protected boolean _equals(StyleBase e) {
		final StylePageLayout other = (StylePageLayout)e;
		if(!pageLayoutProperties.equals(other.pageLayoutProperties)) {
			return false;
		}
		if(!headerProperties.equals(other.headerProperties)) {
			return false;
		}
		if(!footerProperties.equals(other.footerProperties)) {
			return false;
		}
		return true;
	}

	@Override
	public StylePageLayout clone() {
		final StylePageLayout clone = (StylePageLayout)_clone();
		clone.pageLayoutProperties = pageLayoutProperties.clone();
		clone.headerProperties = headerProperties.clone();
		clone.footerProperties = footerProperties.clone();
		return clone;
	}
}
