/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016-2020 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.office.filter.ooxml.pptx;

import java.io.InputStream;
import org.pptx4j.jaxb.Context;
import com.openexchange.office.filter.api.DocumentProperties;
import com.openexchange.office.filter.api.FilterException;
import com.openexchange.office.filter.api.FilterException.ErrorCode;
import com.openexchange.office.filter.api.IExporter;
import com.openexchange.office.filter.ooxml.OfficeOpenXMLOperationDocument;
import com.openexchange.office.imagemgr.IResourceManager;
import com.openexchange.session.Session;

/**
 * {@link Exporter}
 *
 * @author <a href="mailto:kai.ahrens@open-xchange.com">Kai Ahrens</a>
 */
public class Exporter implements IExporter {

    /**
     * Initializes a new {@link Exporter}.
     */
    public Exporter() {
        Context.getpmlObjectFactory();
    }

    /*
     * (non-Javadoc)
     * @see com.openexchange.office.IExporter#createDocument(java.io.InputStream, org.json.JSONObject)
     */
    @Override
    public InputStream createDocument(Session session, InputStream documentStm, String applyOperations, IResourceManager resourceManager, DocumentProperties documentProperties, boolean createFinalDocument) {
        try {
            return _createDocument(session, documentStm, applyOperations, resourceManager, documentProperties, createFinalDocument, true);
        }
        catch(FilterException e) {
              if(e.getErrorcode()!=ErrorCode.CRITICAL_ERROR) {
                  e.setOperationCount(0);
                  throw e;
              }
        }
        try {
            documentStm.reset();
        }
        catch(Exception e) {
            OfficeOpenXMLOperationDocument.rethrowFilterException(e, null);
        }
        // fallback, try without sorting operations
        return _createDocument(session, documentStm, applyOperations, resourceManager, documentProperties, createFinalDocument, false);
    }

    public InputStream _createDocument(Session session, InputStream documentStm, String applyOperations, IResourceManager resourceManager, DocumentProperties documentProperties, boolean createFinalDocument, boolean sortOperations) {
        InputStream document = null;
        try(PptxOperationDocument operationDocument = new PptxOperationDocument(session, resourceManager, documentProperties)) {
            operationDocument.loadDocument(documentStm, false);
            operationDocument.setSortOperations(sortOperations);
            operationDocument.setCreateFinalDocument(createFinalDocument);
            operationDocument.updateDocumentProperties();
            operationDocument.applyOperations(applyOperations);
            document = operationDocument.debugSave(applyOperations);
        }
        catch(Throwable e) {
            OfficeOpenXMLOperationDocument.rethrowFilterException(e, null);
        }
        return document;
    }
}
