/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the OX Software GmbH group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016-2020 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.office.rt2.exception;

import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.apache.commons.text.TextStringBuilder;

import com.openexchange.office.rt2.core.RT2LogInfo;
import com.openexchange.office.tools.error.ErrorCode;

//=============================================================================
public class RT2TypedException extends RT2Exception
{
	//-------------------------------------------------------------------------
	private static final long serialVersionUID = 1L;

	//-------------------------------------------------------------------------
	private final ErrorCode m_aError;

	//-------------------------------------------------------------------------
	public RT2TypedException (final ErrorCode aError, List<RT2LogInfo> msgs)
	{
	    super (getTextFromErrorCode(aError) + " " + format(msgs));
	    m_aError = (null == aError) ? ErrorCode.GENERAL_UNKNOWN_ERROR : aError;
	}

    //-------------------------------------------------------------------------
    public RT2TypedException (final ErrorCode aError, List<RT2LogInfo> msgs, String ... additionalInfo)
    {
        super (getTextFromErrorCode(aError) + format(additionalInfo)  + " " + format(msgs));
        m_aError = (null == aError) ? ErrorCode.GENERAL_UNKNOWN_ERROR : aError;
    }

	//-------------------------------------------------------------------------
	public RT2TypedException (final ErrorCode aError, List<RT2LogInfo> msgs, final Throwable aCause)
	{
		super (getTextFromErrorCode(aError) + " " + format(msgs), aCause);
		m_aError = (null == aError) ? ErrorCode.GENERAL_UNKNOWN_ERROR : aError;
	}

	//-------------------------------------------------------------------------
	public ErrorCode getError ()
	{
		return m_aError;
	}

	//-------------------------------------------------------------------------
	private static String getTextFromErrorCode(final ErrorCode aErrorCode)
	{
		if (null == aErrorCode) {
			return ErrorCode.GENERAL_UNKNOWN_ERROR.getDescription();
		}
		return (StringUtils.isEmpty(aErrorCode.getDescription())) ? aErrorCode.getCodeAsStringConstant() : aErrorCode.getDescription();
	}

	private static String format(List<RT2LogInfo> msgs) {
		if ((msgs != null) && !msgs.isEmpty()) {
			TextStringBuilder strBuilder = new TextStringBuilder();
			RT2LogInfo logInfo = msgs.get(0);
			if (logInfo.isDocProcessorLogInfo()) {
				strBuilder.append("Current messages of DocProcessor with docUid '");
				strBuilder.append(logInfo.getDocUid());					 
			} else {
				 strBuilder.append("Current messages of DocProxy with docUid '");
				 strBuilder.append(logInfo.getDocUid());
				 strBuilder.append("' and clientUid '");
				 strBuilder.append(logInfo.getClientUid());
			}
			strBuilder.append("': ");
			strBuilder.appendNewLine();
			msgs.forEach(s -> strBuilder.appendln("    " + s));		    				 
			return strBuilder.toString();
		}
		return "";
	}
	

	
	//-------------------------------------------------------------------------
	private static String format(String [] strArray) {
	    StringBuilder strBuilder = new StringBuilder();
	    for (int i=0;i+1<strArray.length;i+=2) {
	        strBuilder.append(strArray[i]);
	        strBuilder.append(":");
	        strBuilder.append(strArray[i+1]);
	        strBuilder.append(";");
	    }
	    return strBuilder.toString();
	}
}
